<?php

namespace Modules\Cadastros\Controllers;
//@Author Yago Ferreira	

use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Doctrine\ORM\EntityManager;
use PHPExcel_IOFactory;


class CadastroFotoSupplyTekController extends \Zeedhi\Framework\Controller\Report {
    
    protected $service;
    protected $environment;
    protected $crudService;
    protected $dataSourceManager;
    protected $entityManager;
    protected $connection;
    protected $fileServerURL;
    protected $imgPathURL;
    protected $zhFileUploader;
    protected $CadastroFotoSupplyTekService;
    protected $uploadRoutes;

    
    public function __construct($service, $environment, $crudService, $dataSourceManager ,EntityManager $entityManager, $connection, $fileServerURL,$imgPathURL,$zhFileUploader, $CadastroFotoSupplyTekService, $uploadRoutes) {
  
		$this->service = $service;
		$this->environment  = $environment;
		$this->crudService  = $crudService;
        $this->dataSourceManager = $dataSourceManager;
        $this->entityManager = $entityManager;
        $this->connection   = $connection;
        $this->fileServerURL     = $fileServerURL;
        $this->imgPathURL = $imgPathURL;
        $this->zhFileUploader    = $zhFileUploader;
        $this->CadastroFotoSupplyTekService      = $CadastroFotoSupplyTekService;
        $this->uploadRoutes     = $uploadRoutes;
                         
        
	}

	
	public function produtosPorFotoSupplyTek(DTO\Request\Filter $request, DTO\Response $response) {
		$filterCriteria = $request->getFilterCriteria();
		$filterCriteria->setDataSourceName("produtosPorFotoSupplyTek");
		$filterCriteria = $this->crudService->getFilterCriteriaWithNrOrg($filterCriteria);
		$newFilterCriteria = $this->crudService->getPopulatedFilterCriteria($filterCriteria, array());
		$dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
		$newDataSet = $this->setImgFullPath($dataSet);
        $response->addDataSet($newDataSet);
       
	}
	
	private function setImgFullPath($dataSet) {
	    
	 
		$rows = $dataSet->getRows();
		foreach ($rows as &$row) {
			if ($row["FOTOPROD"] !== null) {
				$row["FOTOPROD"] = $this->imgPathURL.$row["FOTOPROD"];
			}
		}
		return new DataSet("produtosPorFotoSupplyTek", $rows);
	}

	public function saveImgDescriptionProd(DTO\Request $request, DTO\Response $response) {
	    
		$row = $request->getRow();
		
		try {
            $this->entityManager->beginTransaction();
		    if($row['FOTOPROD'] != null){
		        
		        if(isset($row["IMAGE_UPLOAD"])){
		            		
        	        $fileRow['UPLOAD_AUT_IMAGE'][0] = [
                        'b64File'            => $row["IMAGE_UPLOAD"][0]["b64File"],
                        "name"               => $row["IMAGE_UPLOAD"][0]["name"],
                        'type'               => $row["IMAGE_UPLOAD"][0]["type"],
                        'webkitRelativePath' => '',
                        'size'               => ''
                    ];
          	        $this->zhFileUploader->uploadFilesInRow($fileRow, $this->uploadRoutes['/savePhotosAutomatic']['fileField'], $this->uploadRoutes['/savePhotosAutomatic']['customFolder']);
    		        $row["DSPATHFOTO"] = $fileRow['UPLOAD_AUT_IMAGE']["0"]["path"];
    		        $fotoProd = $this->imgPathURL.$row["DSPATHFOTO"];

		        }
		        else{
    		        $fotoProd = $row['FOTOPROD'];
		        }
		    }else{
		        $row["DSPATHFOTO"] = null;
		        $fotoProd = $row['FOTOPROD'];
		    }
		   
		  // Verifica se existe produto cadastrado na tabela PRODSUPPLYTEK, caso n existe faz um insert;
		    $this->existeProdutoSupplyTek($row);
		    
		    
    		$this->CadastroFotoSupplyTekService->saveImgDescriptionProd($row);
    		$dataset = array(
                "PATHIMAGE"   => $fotoProd,
                "DSDESCRICAO"   => $row['DSDESCRICAO']
            );
            $response->addDataSet(new DataSet('data', $dataset));
		    $this->entityManager->commit();
        } catch (\Exception $e){
            $this->entityManager->rollBack();
            throw $e;
        }
	}

    
    public function existeProdutoSupplyTek ($row) {
         
        $sql = "SELECT CDPRODUTO FROM PRODSUPPLYTEK WHERE CDPRODUTO =:CDPRODUTO";
        $params = array(
            "CDPRODUTO"   =>  $row['CDPRODUTO']
        );
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        $product = $std->fetchAll();
        
        // Caso o CDPRODUTO não retornou da tabela PRODSUPPLYTEK, faz o insert dele.
        if(empty($product)){
        $this->insertProdutoSupplyTek($row["CDPRODUTO"]);
        }
    }
     /* ***************
     *    INSERT     *
     *****************/
    public function insertProdutoSupplyTek($cdproduto) {
        $params = array(
		    "CDPRODUTO" => $cdproduto
		);

        $query = 'INSERT INTO PRODSUPPLYTEK (IDPRODSUPPLYTEK, CDPRODUTO)
                    VALUES (SEQ_PRODSUPPLYTEK.NEXTVAL, :CDPRODUTO)';
	    $stmt = $this->connection->prepare($query);
		$stmt->execute($params);
        
    }
    

	
}