var ZeedhiDirectives = angular.module("ZeedhiDirectives");

ZeedhiDirectives.directive('zhScroller', ["$timeout", "templateManager", "ScreenService", "ZhLogger", function($timeout, templateManager, ScreenService, zhLogger) {
	return {
		restrict: 'A',
		link: function($scope, iElement, iAttrs) {
			var expectedHeight = 0;
			var hasAttr = function hasAttr(element, attr) {
				return (typeof element.attr(attr) !== "undefined" && element.attr(attr) !== false) || (typeof element.attr('data-' + attr) !== "undefined" &&
					element.attr('data-' + attr) !== false);
			};

			//Zed - adjust top and bottom dinamically
			var startScroller = function(element) {
				element = $(element);
				if (element.is(":visible")) {

					// reset css position attributes
					element.css({
						'position': '',
						'top': '',
						'bottom': ''
					});

					var windowHeight = $(window).height();
					var containerTop = element.offset().top;
					var containerBottom = 0;
					// if container has margin-top then adjust the top
					var containerMarginTop = parseFloat(element.css('margin-top') || window.getComputedStyle(element[0], false).marginTop);
					containerTop -= containerMarginTop;

					// defines the maximum height from the container. At least 260px and maximum of half window
					var maxContainerHeight = Math.ceil(windowHeight * 0.5) > 260 ? Math.ceil(windowHeight * 0.5) : 260;

					// verifies if his element has a container fixed on the bottom of page and adjusts the bottom
					var parent = element.parent();
					var fixedBottomContainer = parent.children(".container[zh-fixed-bottom]")[0];
					if (fixedBottomContainer) {
						containerBottom = fixedBottomContainer.clientHeight;
					}

					if (element.hasClass("has-stripe-footer")) {
						containerBottom += Util.isMobile() ? 50 : 40; // stripe-footer height
					}

					// search for a parent container with position non static or another container[zh-scroller]
					while (parent && parent.css('position') === 'static' &&
						!(parent.hasClass('container') && hasAttr(parent, 'zh-scroller'))) {
						parent = parent.parent();
					}

					// if there is a parent non static then adjusts the top
					if (parent && parent.offset) {
						containerTop = containerTop - parent.offset().top;
					}

					var popup = element.closest('.popup-content').length > 0;
					var parentClass = element.parent().attr('class');
					var isCalendarWidget = parentClass && parentClass.includes('zh-calendar-widget');

					if (!popup) {
						if (!isCalendarWidget) {
							element.css('position', 'absolute');
						}
						element.css('top', containerTop + 'px');
						element.css('bottom', containerBottom + 'px');

					} else {

						element.css('top', '0px');
						element.css('width', 'auto');
						element.css('position', 'relative');

						if (element.find('.zh-grid-container').length > 0 ||
							element.find('.zh-list-container').length > 0) {
							element.css('overflow', 'hidden');
						} else {
							element.css('overflow', 'auto');
						}

						var calendar, calendarHeight;

						if (element.is(".zh-carousel") && element.parents(".zh-interval-widget").length) {
							calendar = element.find(".zh-calendar-widget");
							calendarHeight = calendar.find('.calendar-navigation').height() + calendar.find('.calendar').height() + 12;
							element.height(Math.min(calendarHeight, 0.9 * window.innerHeight - 150));
						} else if ((element.find('.zh-template-list').length > 0 || element.parents(".zh-carousel").length > 0) && element.parents(".zh-interval-widget").length) {
							var interval = element.parents(".zh-interval-widget");
							calendar = interval.find(".zh-calendar-widget");
							calendarHeight = calendar.find('.calendar-navigation').height() + calendar.find('.calendar').height() + 12;
							element.height(Math.min(calendarHeight, 0.9 * window.innerHeight - 150));
						} else if (
							element.find('.zh-grid, .zh-new-grid').length > 0 ||
							element.find('.zh-template-list').length > 0 ||
							element.find('.zh-tab-wrapper').length > 0) {

							//temp-fix: sometimes element.offset().top can be negative and it mess things up
							//causing anomalies with the select poupus
							if (containerTop < 0) {
								containerTop = 0;
							}

							if (containerTop === 0) {
								containerTop = getContainerTopForListInsideTabbed(element, containerTop);
							}

							// Lists, grids and tree-grids need a fixed height so you can scroll
							if (!Util.isUndefined(element.parent().attr('data-zh-scroller')) || !Util.isUndefined(element.parent().attr('zh-scroller'))) {
								element.css('height', '100%');
							} else {
								element.css('height', (maxContainerHeight - containerTop) + 'px');
							}
						} else if (Zepto.os.phone) {
							element.css('max-height', (0.9 * window.innerHeight - 100) + 'px');
						} else if (!element.is('[zh-calendar]')) {
							element.css('max-height', (maxContainerHeight - containerTop) + 'px');
						}
					}

					expectedHeight = Zepto.os.phone ? (0.9 * window.innerHeight - 100) : (maxContainerHeight - containerTop);

					// prevent flickering due the reverse order of execution of this function thru the DOM tree
					element.animate({
						opacity: 1
					}, 250);

					if (Util.isMobile()) {
						// prevent bounce when scrolling outside div
						var elementToApplyNonbounce = iAttrs.zhScrollerNonbounceSelector ? element.find(iAttrs.zhScrollerNonbounceSelector) : element;
						if (elementToApplyNonbounce) {
							elementToApplyNonbounce.nonbounce();
						} else {
							zhLogger.warn("nonbouneSelector \"" + iAttrs.nonbounceSelector + "\" was not found!");
						}
					}

					$scope.$broadcast("zh-scrolled-finished", element);
				}
			};

			var unbindWatch = $scope.$watch(function() {
				return iElement.is(':visible');
			}, function(value) {
				if (value === true) {
					unbindWatch();

					iElement[0].startScroller = startScroller;
					$timeout(function() {
						startScroller(iElement);
					}, 250, false);
				}
			});

			$scope.$on('$destroy', function() {
				delete iElement[0].startScroller;
				$(window).off('orientationchange.' + $scope.$id);
			});

			if (Util.isMobile()) {
				$scope.$watch(function() {
					return ScreenService.isKeyboardOpen() && ScreenService.isPopupOpen();
				}, function(newValue, oldValue) {
					$timeout(function() {
						if (ScreenService.isPopupOpen() && ScreenService.getPopupOpen() == $scope.widget) {
							$(window).on('orientationchange.' + $scope.$id, function(){
								$timeout(function() {
									if(window.matchMedia("(orientation: landscape)").matches){
										iElement.closest('.popup-content').find('[zh-scroller], [data-zh-scroller]').each(function(index, item) {
											startScroller($(item));
										});
									}
								}, 400, false);
							});
							var popupScroller = iElement.parent("[zh-scroller]");
							if (!popupScroller.length) {
								popupScroller = iElement.parent("[data-zh-scroller]");
							}

							if (popupScroller.length) {
								if (expectedHeight > window.innerHeight - 100) {
									popupScroller.css({
										"height": window.innerHeight - 100
									});
									iElement.css({
										"height": window.innerHeight - 100
									});
								} else {
									popupScroller.css({
										"height": expectedHeight
									});
									iElement.css({
										"height": expectedHeight
									});
								}
							}
						}
					}, 0, false);
				});
			}
		},

		priority: 10
	};

	function getContainerTopForListInsideTabbed(element, containerTop) {
		var tabbedContainer = element.parents(".container.zh-tabbed-container");
		var tabbedHeader = tabbedContainer.find(".zh-header.zh-custom-tabbed-widget");
		if (tabbedHeader.length) {
			containerTop = tabbedHeader.outerHeight();
		}

		return containerTop;
	}
}]);